/*
 * Copyright (c) 2021 Ubique Innovation AG <https://www.ubique.ch>
 *
 *  This Source Code Form is subject to the terms of the Mozilla Public
 *  License, v. 2.0. If a copy of the MPL was not distributed with this
 *  file, You can obtain one at https://mozilla.org/MPL/2.0/.
 *
 *  SPDX-License-Identifier: MPL-2.0
 */

import MapCoreSharedModule
import MetalKit

struct PolygonVertex: Equatable {
    var position: SIMD2<Float>

    var stylingIndex: Int32 = 0

    /// Returns the descriptor to use when passed to a metal shader
    static let descriptor: MTLVertexDescriptor = {
        let vertexDescriptor = MTLVertexDescriptor()
        var offset = 0
        let bufferIndex = 0

        // Position
        vertexDescriptor.attributes[0].bufferIndex = bufferIndex
        vertexDescriptor.attributes[0].format = .float2
        vertexDescriptor.attributes[0].offset = offset
        offset += MemoryLayout<SIMD2<Float>>.stride

        // Styling Index
        vertexDescriptor.attributes[1].bufferIndex = bufferIndex
        vertexDescriptor.attributes[1].format = .int
        vertexDescriptor.attributes[1].offset = offset
        offset += MemoryLayout<Int>.stride

        vertexDescriptor.layouts[0].stride = MemoryLayout<PolygonVertex>.stride
        return vertexDescriptor
    }()

    init(x: Float,
         y: Float,
         stylingIndex: Int32 = 0) {
        position = SIMD2([x, y])
        self.stylingIndex = stylingIndex
    }
}
